/*
  List.c
  ------

  Stewart's Linked List Code.
  (C)2001-2003 Stewart Smith
  Distributed under the GNU Public License
  (see included LICENSE file for details)

  This source file implements a linked list of void pointers.
  That is, these can be pointers to *anything* and it is up to the
  programmer to remember what types are where and to do the correct
  typecasting where appropriate.
  This is the type of code that you wish was in Objective-C so you could
  just use the id type (or just use NSMutableArray).

  Needs reworking so it malloc's itself, and then frees itself... but that's for later.
  Documentation is in List.h in HeaderDoc format.
 */

#include <stdlib.h>
#include "List.h"

int list_new(List *theList)
{
	theList->numItems = 0;
	theList->list = NULL;
	theList->current = NULL;
	theList->freeItem = NULL;
        return 1;
}

void list_destroy(List *theList)
{
	ListNode *current = theList->list;

	while(current!=NULL)
	{
		theList->list = current->next;
		if(theList->freeItem)
		  theList->freeItem(current->item);
		free (current);
		current = theList->list;
    }

}

int list_append(List *theList,void* theItem)
{
	ListNode *current = theList->list;

	if(theList->list == NULL) // actually theList
	{
		theList->list = (ListNode*)malloc(sizeof(ListNode));
                if(theList->list == NULL)
                    return 0;
		theList->list->item = theItem;
		theList->list->next = NULL;
	}
	else
	{
		while(current->next!=NULL)
			current = current->next;
		current->next = (ListNode*)malloc(sizeof(ListNode));
                if (current->next==NULL)
                    return 0;
		current->next->item = theItem;
		current->next->next = NULL;
	}
	theList->numItems++;
        return 1;
}

void *list_getItem(List *theList,void* item, int (*compare)(void*,void*))
{
    ListNode *current = theList->list;
    
    if(theList->list ==NULL)
        return 0;
    
    while(current!=NULL)
    {
        if(compare(current->item,item))
            return current->item;
        current = current->next;
    }
    return 0;
}

void *list_getID(List *theList,long id)
{
    ListNode *current = theList->list;
    long i;

    if(theList->list == NULL)
        return 0;
    
    for(i=0;i<id;current=current->next,i++)
        if(current->next==NULL)
	  return 0;
        
    return current->item;
}

void* list_traverse(List *theList, ListNode** current)
{
  if(*current == NULL)
    *current = theList->list;
  else
    *current = (*current)->next;

  if (*current !=NULL)
      return (*current)->item;
  else
      return NULL;
}
